@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2012, 2014-2017 Free Software Foundation,
-   Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -136,13 +135,14 @@ mail_sendheader (int argc, char **argv)
 /* Attachments */
 struct atchinfo
 {
-  char *id;
-  char *encoding;
-  char *content_type;
-  char *name;
-  char *filename;
-  mu_stream_t source;
-  int skip_empty;
+  char *id;            /* Attachment id (for listing) */
+  char *encoding;      /* Encoding the attachment uses */
+  char *content_type;  /* Full content type (type/subtype[; attrs]) */
+  char *name;          /* Attachment name */
+  char *filename;      /* Attachment file name */
+  mu_stream_t source;  /* Attachment source stream */
+  int skip_empty;      /* Skip this attachment if it is empty */
+  int disp_inline;     /* Inline content disposition */
 };
 
 static void
@@ -195,7 +195,7 @@ static void
 attlist_add (mu_list_t attlist, char *id, char const *encoding,
 	     char const *content_type, char const *content_name,
 	     char const *content_filename,
-	     mu_stream_t stream, int skip_empty)
+	     mu_stream_t stream, int skip_empty, int disp_inline)
 {
   struct atchinfo *aptr;
   int rc;
@@ -213,6 +213,7 @@ attlist_add (mu_list_t attlist, char *id, char const *encoding,
   if (stream)
     mu_stream_ref (stream);
   aptr->skip_empty = skip_empty;
+  aptr->disp_inline = disp_inline;
   rc = mu_list_append (attlist, aptr);
   if (rc)
     {
@@ -312,7 +313,7 @@ attlist_attach_file (mu_list_t *attlist_ptr,
     
   attlist_add (attlist, id, encoding, content_type,
 	       content_name, content_filename,
-	       stream, skip_empty_attachments);
+	       stream, skip_empty_attachments, 0);
   if (stream)
     mu_stream_unref (stream);
   free (id);
@@ -326,7 +327,8 @@ attlist_helper (void *item, void *data)
   struct atchinfo *aptr = item;
   mu_list_t list = data;
   attlist_add (list, aptr->id, aptr->encoding, aptr->content_type,
-	       aptr->name, aptr->filename, aptr->source, aptr->skip_empty);
+	       aptr->name, aptr->filename, aptr->source, aptr->skip_empty,
+	       aptr->disp_inline);
   return 0;
 }
 
@@ -453,26 +455,13 @@ escape_remove_attachment (int argc, char **argv, compose_env_t *env)
 }
 
 static int
-saveatt (void *item, void *data)
+save_attachment (struct atchinfo *aptr, compose_env_t *env, mu_message_t part)
 {
-  struct atchinfo *aptr = item;
-  compose_env_t *env = data;
-  mu_message_t part;
   mu_header_t hdr;
   int rc;
   size_t nparts;
   char *p;
-
-  rc = mu_attachment_create (&part, aptr->content_type, aptr->encoding,
-			     aptr->name,
-			     aptr->filename);
-  if (rc)
-    {
-      mu_error (_("can't create attachment %s: %s"),
-		aptr->id, mu_strerror (rc));
-      return 1;
-    }
-
+  
   rc = mu_attachment_copy_from_stream (part, aptr->source);
   if (rc)
     {
@@ -507,8 +496,17 @@ saveatt (void *item, void *data)
   mu_header_set_value (hdr, MU_HEADER_CONTENT_ID, p, 1);
   free (p);
 
+  if (aptr->disp_inline)
+    {
+      rc = mu_header_set_value (hdr, MU_HEADER_CONTENT_DISPOSITION,
+				"inline",
+				1);
+      if (rc)
+	mu_diag_funcall (MU_DIAG_ERROR, "mu_header_set_value",
+			 MU_HEADER_CONTENT_DISPOSITION, rc);
+    }
+  
   rc = mu_mime_add_part (env->mime, part);
-  mu_message_unref (part);
   if (rc)
     {
       mu_diag_funcall (MU_DIAG_ERROR, "mu_mime_add_part", aptr->filename, rc);
@@ -518,6 +516,29 @@ saveatt (void *item, void *data)
   return 0;
 }
 
+static int
+saveatt (void *item, void *data)
+{
+  struct atchinfo *aptr = item;
+  compose_env_t *env = data;
+  mu_message_t part;
+  int rc;
+  
+  rc = mu_attachment_create (&part, aptr->content_type, aptr->encoding,
+			     aptr->name,
+			     aptr->filename);
+  if (rc)
+    {
+      mu_error (_("can't create attachment %s: %s"),
+		aptr->id, mu_strerror (rc));
+      return 1;
+    }
+  
+  rc = save_attachment (aptr, env, part);
+  mu_message_unref (part);
+  return rc;
+}
+
 static int
 add_body (mu_message_t inmsg, compose_env_t *env)
 {
@@ -530,13 +551,14 @@ add_body (mu_message_t inmsg, compose_env_t *env)
   mu_body_get_streamref (body, &str);
 
   aptr = mu_alloc (sizeof (*aptr));
-  aptr->id = NULL;
+  mu_asprintf (&aptr->id, "(body)");
   aptr->encoding = default_encoding ? mu_strdup (default_encoding) : NULL;
   attach_set_content_type (aptr, default_content_type);
   aptr->name = NULL;
   aptr->filename = NULL;
   aptr->source = str;
   aptr->skip_empty = skip_empty_attachments || multipart_alternative;
+  aptr->disp_inline = 1;
   if (!env->attlist)
     env->attlist = attlist_new ();
   rc = mu_list_prepend (env->attlist, aptr);
@@ -544,7 +566,7 @@ add_body (mu_message_t inmsg, compose_env_t *env)
     mu_diag_funcall (MU_DIAG_ERROR, "mu_list_prepend", NULL, rc);
   return rc;
 }
-  
+
 static int
 add_attachments (compose_env_t *env, mu_message_t *pmsg)
 {
@@ -555,7 +577,7 @@ add_attachments (compose_env_t *env, mu_message_t *pmsg)
   
   inmsg = *pmsg;
 
-  if (mailvar_is_true ("mime") && add_body (inmsg, env))
+  if (mailvar_is_true (mailvar_name_mime) && add_body (inmsg, env))
     return 1;
   
   if (mu_list_is_empty (env->attlist))
@@ -638,10 +660,10 @@ add_attachments (compose_env_t *env, mu_message_t *pmsg)
 static void
 read_cc_bcc (compose_env_t *env)
 {
-  if (mailvar_is_true ("askcc"))
+  if (mailvar_is_true (mailvar_name_askcc))
     compose_header_set (env, MU_HEADER_CC,
 			ml_readline_with_intr ("Cc: "), COMPOSE_REPLACE);
-  if (mailvar_is_true ("askbcc"))
+  if (mailvar_is_true (mailvar_name_askbcc))
     compose_header_set (env, MU_HEADER_BCC,
 			ml_readline_with_intr ("Bcc: "), COMPOSE_REPLACE);
 }
@@ -674,7 +696,7 @@ mail_send (int argc, char **argv)
       if (interactive)
 	compose_header_set (&env, MU_HEADER_TO, ml_readline_with_intr ("To: "),
 			    COMPOSE_REPLACE);
-      else if (mailvar_is_true ("editheaders"))
+      else if (mailvar_is_true (mailvar_name_editheaders))
 	{
 	  if (parse_headers (mu_strin, &env) != parse_headers_ok)
 	    {
@@ -729,10 +751,10 @@ mail_send (int argc, char **argv)
 
   if (interactive)
     {
-      if (!mailvar_is_true ("mailx"))
+      if (!mailvar_is_true (mailvar_name_mailx))
 	read_cc_bcc (&env);
 
-      if (mailvar_is_true ("asksub"))
+      if (mailvar_is_true (mailvar_name_asksub))
 	compose_header_set (&env, MU_HEADER_SUBJECT,
 			    ml_readline_with_intr ("Subject: "),
 			    COMPOSE_REPLACE);
@@ -878,7 +900,8 @@ compose_header_set (compose_env_t *env, const char *name,
   switch (mode)
     {
     case COMPOSE_REPLACE:
-      if (is_address_field (name) && mailvar_is_true ("inplacealiases"))
+      if (is_address_field (name)
+	  && mailvar_is_true (mailvar_name_inplacealiases))
 	{
 	  char *exp = alias_expand (value);
 	  status = mu_header_set_value (env->header, name, exp ? exp : value, 1);
@@ -889,7 +912,8 @@ compose_header_set (compose_env_t *env, const char *name,
       break;
 
     case COMPOSE_APPEND:
-      if (is_address_field (name) && mailvar_is_true ("inplacealiases"))
+      if (is_address_field (name)
+	  && mailvar_is_true (mailvar_name_inplacealiases))
 	{
 	  char *exp = alias_expand (value);
 	  status = mu_header_append (env->header, name, exp ? exp : value);
@@ -903,7 +927,8 @@ compose_header_set (compose_env_t *env, const char *name,
       if (mu_header_aget_value (env->header, name, &old_value) == 0
 	  && old_value[0])
 	{
-	  if (is_address_field (name) && mailvar_is_true ("inplacealiases"))
+	  if (is_address_field (name)
+	      && mailvar_is_true (mailvar_name_inplacealiases))
 	    {
 	      char *exp = alias_expand (value);
 	      status = util_merge_addresses (&old_value, exp ? exp : value);
@@ -935,12 +960,12 @@ compose_header_set (compose_env_t *env, const char *name,
   return status;
 }
 
-char *
+char const *
 compose_header_get (compose_env_t *env, char *name, char *defval)
 {
-  char *p;
+  char const *p;
 
-  if (mu_header_aget_value (env->header, name, &p))
+  if (mu_header_sget_value (env->header, name, &p))
     p = defval;
   return p;
 }
@@ -987,10 +1012,11 @@ fill_body (mu_message_t msg, mu_stream_t instr)
   
   if (n == 0)
     {
-      if (mailvar_is_true ("nullbody"))
+      if (mailvar_is_true (mailvar_name_nullbody))
 	{
 	  char *str;
-	  if (mailvar_get (&str, "nullbodymsg", mailvar_type_string, 0) == 0)
+	  if (mailvar_get (&str, mailvar_name_nullbodymsg,
+			   mailvar_type_string, 0) == 0)
 	    mu_error ("%s", _(str));
 	}
       else
@@ -1003,7 +1029,7 @@ fill_body (mu_message_t msg, mu_stream_t instr)
 static int
 save_dead_message_env (compose_env_t *env)
 {
-  if (mailvar_is_true ("save"))
+  if (mailvar_is_true (mailvar_name_save))
     {
       mu_stream_t dead_letter, str;
       int rc;
@@ -1021,7 +1047,7 @@ save_dead_message_env (compose_env_t *env)
 	  mu_error (_("Cannot open file %s: %s"), name, strerror (rc));
 	  return 1;
 	}
-      if (mailvar_is_true ("appenddeadletter"))
+      if (mailvar_is_true (mailvar_name_appenddeadletter))
 	mu_stream_seek (dead_letter, 0, MU_SEEK_END, NULL);
       else
 	mu_stream_truncate (dead_letter, 0);
@@ -1054,7 +1080,7 @@ save_dead_message_env (compose_env_t *env)
 static int
 save_dead_message (mu_message_t msg)
 {
-  if (mailvar_is_true ("save"))
+  if (mailvar_is_true (mailvar_name_save))
     {
       mu_stream_t dead_letter, str;
       int rc;
@@ -1072,7 +1098,7 @@ save_dead_message (mu_message_t msg)
 	  mu_error (_("Cannot open file %s: %s"), name, strerror (rc));
 	  return 1;
 	}
-      if (mailvar_is_true ("appenddeadletter"))
+      if (mailvar_is_true (mailvar_name_appenddeadletter))
 	mu_stream_seek (dead_letter, 0, MU_SEEK_END, NULL);
       else
 	mu_stream_truncate (dead_letter, 0);
@@ -1103,7 +1129,8 @@ send_message (mu_message_t msg)
   char *sendmail;
   int status;
   
-  if (mailvar_get (&sendmail, "sendmail", mailvar_type_string, 0) == 0)
+  if (mailvar_get (&sendmail, mailvar_name_sendmail,
+		   mailvar_type_string, 0) == 0)
     {
       if (sendmail[0] == '/')
 	status = msg_to_pipe (sendmail, msg);
@@ -1117,7 +1144,7 @@ send_message (mu_message_t msg)
 	      const char *return_address_str;
 	      mu_address_t return_address = NULL;
 	      
-	      if (mailvar_get (&return_address_str, "return-address",
+	      if (mailvar_get (&return_address_str, mailvar_name_return_address,
 			       mailvar_type_string, 0) == 0)
 		{
 		  status = mu_address_create (&return_address,
@@ -1131,7 +1158,7 @@ send_message (mu_message_t msg)
 		    }
 		} 
 
-	      if (mailvar_is_true ("verbose"))
+	      if (mailvar_is_true (mailvar_name_verbose))
 		{
 		  mu_debug_set_category_level (MU_DEBCAT_MAILER,
 					  MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
@@ -1166,7 +1193,7 @@ send_message (mu_message_t msg)
    If the variable "record" is set, the outgoing message is
    saved after being sent. If "save_to" argument is non-zero,
    the name of the save file is derived from "to" argument. Otherwise,
-   it is taken from the value of "record" variable.
+   it is taken from the value of the "record" variable.
 
    sendmail
 
@@ -1202,14 +1229,14 @@ mail_send0 (compose_env_t *env, int save_to)
 
       if (ml_got_interrupt ())
 	{
-	  if (mailvar_is_true ("ignore"))
+	  if (buf)
+	    free (buf);
+	  if (mailvar_is_true (mailvar_name_ignore))
 	    {
 	      mu_printf ("@\n");
 	    }
 	  else
 	    {
-	      if (buf)
-		free (buf);
 	      if (++int_cnt == 2)
 		break;
 	      mu_error (_("\n(Interrupt -- one more to kill letter)"));
@@ -1219,9 +1246,9 @@ mail_send0 (compose_env_t *env, int save_to)
 
       if (!buf)
 	{
-	  if (interactive && mailvar_is_true ("ignoreeof"))
+	  if (interactive && mailvar_is_true (mailvar_name_ignoreeof))
 	    {
-	      mu_error (mailvar_is_true ("dot") 
+	      mu_error (mailvar_is_true (mailvar_name_dot) 
                           ?  _("Use \".\" to terminate letter.") 
                           : _("Use \"~.\" to terminate letter."));
 	      continue;
@@ -1232,9 +1259,10 @@ mail_send0 (compose_env_t *env, int save_to)
 
       int_cnt = 0;
 
-      if (strcmp (buf, ".") == 0 && mailvar_is_true ("dot"))
+      if (strcmp (buf, ".") == 0 && mailvar_is_true (mailvar_name_dot))
 	done = 1;
-      else if (mailvar_get (&escape, "escape", mailvar_type_string, 0) == 0
+      else if (mailvar_get (&escape, mailvar_name_escape,
+			    mailvar_type_string, 0) == 0
 	       && buf[0] == escape[0])
 	{
 	  if (buf[1] == buf[0])
@@ -1288,11 +1316,11 @@ mail_send0 (compose_env_t *env, int save_to)
 
   /* In mailx compatibility mode, ask for Cc and Bcc after editing
      the body of the message */
-  if (mailvar_is_true ("mailx"))
+  if (mailvar_is_true (mailvar_name_mailx))
     read_cc_bcc (env);
 
   /* Prepare the header */
-  if (mailvar_is_true ("xmailer"))
+  if (mailvar_is_true (mailvar_name_xmailer))
     mu_header_set_value (env->header, MU_HEADER_X_MAILER, program_version, 1);
 
   if (util_header_expand (&env->header) == 0)
@@ -1324,17 +1352,20 @@ mail_send0 (compose_env_t *env, int save_to)
 	  /* Save outgoing message */
 	  if (save_to)
 	    {
-	      char *tmp = compose_header_get (env, MU_HEADER_TO, NULL);
-	      mu_address_t addr = NULL;
-	      
-	      mu_address_create (&addr, tmp);
-	      mu_address_aget_email (addr, 1, &savefile);
-	      mu_address_destroy (&addr);
-	      if (savefile)
+	      char const *rcpt = compose_header_get (env, MU_HEADER_TO, NULL);
+	      if (rcpt)
 		{
-		  char *p = strchr (savefile, '@');
-		  if (p)
-		    *p = 0;
+		  mu_address_t addr = NULL;
+	      
+		  mu_address_create (&addr, rcpt);
+		  mu_address_aget_email (addr, 1, &savefile);
+		  mu_address_destroy (&addr);
+		  if (savefile)
+		    {
+		      char *p = strchr (savefile, '@');
+		      if (p)
+			*p = 0;
+		    }
 		}
 	    }
 	  util_save_outgoing (msg, savefile);
